#! /bin/csh
#
# Magic filter for my SLM printer setup
#

# set path
setenv PATH /usr/bin:/bin:/usr/local/bin:/var/spool/lpd

# get printer and filter name from the name we're called as
set basename   = $0:t
set printer    = $basename:r
set filtertype = $basename:e

# find out if my input's piped - if so I need a tmp file.
if ({ rewind-stdin }) then
	set tmpfile = ""
else
	# get an unused temp file name
	set i = 0
	while ( -f /tmp/$basename.$$.$i )
		@ i = $i + 1
	end
	set tmpfile = /tmp/$basename.$$.$i
endif

# run 'file' on our input -- maybe a short tmp file is necessary if input
# is not seekable
if ( "$tmpfile" == "" ) then
	set magic = `file -`
else
	dd bs=1 count=1024 of=$tmpfile >&/dev/null
	set magic = `file $tmpfile`
endif

# extract the real file type from file's output
set magic = `echo $magic | sed -e 's/.*:[ 	]*\(.*\)/\1/'`

# look if the input is compressed
switch( "$magic" )

  case gzip*:
  case compress*:
	# decompress file first
	# (gzip can also decompressed files compressed by 'compress')
	if ( "$tmpfile" == "" ) then
    	rewind-stdin
		$prefilter | $0 $*
	else
		( cat $tmpfile; rm $tmpfile; cat ) | $prefilter | $0 $*
	fi
	exit

  default:
	breaksw

endsw

# determine the page format (may be needed by a filter)
set paperformat = `head -2 /dev/slm0 | tail -1 | sed -e 's/^.*(\(.*\)).*$/\1/' | tr A-Z a-z`

# ok, now lets look at the real type of our input
# convert first to PostScript somehow (we need gs anyway for formatting it)

set filter = ""
set needseek = 0

switch( "$magic" )

  case PostScipt*:
	# not much to do here...
	set filter = "cat"
	breaksw

  case DVI*:
	# DVI file: use dvips
	set filter = "dvips -q -f -D 300 -t $paperformat"
	set needseek = 1
	breaksw

  case troff*:
	# groff input: format as man page
	set filter = "geqn | gtbl | groff -Tps -mandoc"
	breaksw

  default:
	# default: assume plain text, use a2ps
	set filter = "a2ps -f -nP -8"
	if ( $filtertype == lf ) then
		# print control chars, don't interpret them
		set filter = "$filter -ni -v"
	endif
	if ( $printer =~ *2* ) then
		# 2 pages per sheet
		set filter = "$filter -l -2"
	else
		set filter = "$filter -p -1"
	endif
	breaksw

endsw

set filter = "$filter | lpSLMf"

if ( "$tmpfile" == "" ) then
	rewind-stdin
	eval "exec $filter $*"
else
	# If this filter needs a rewindable STDIN then the whole thing
	# must go in the tmpfile
	if ( $needseek ) then 
		cat >>$tmpfile
		eval $filter $* <$tmpfile
		rm $tmpfile
	else
		( cat $tmpfile; rm $tmpfile; cat ) | eval $filter $*
		exit
	endif
endif
