
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <asm/atari_SLM.h>



/***************************** Prototypes *****************************/

int main( int argc, char *argv[] );
void usage( void );
void set_pagesize( const char *arg );
void get_pagesize( void );
void set_mfeed( const char *arg );
void get_mfeed( void );

/************************* End of Prototypes **************************/



typedef void (*SFUNC)( const char *arg );
typedef void (*RFUNC)( void );

struct cmd_t {
	char	*cmd;
	SFUNC	set;
	RFUNC	report;
} Cmds[] = {
	{ "pagesize", set_pagesize, get_pagesize },
	{ "mfeed", set_mfeed, get_mfeed }
};

#define	N_CMDS	(sizeof(Cmds)/sizeof(*Cmds))

int		PrinterFd;


int main( int argc, char *argv[] )

{	int		arg = 1, i;
	char	*printer = NULL;
	int		report = 0;
	
	/* first arg is "-v" or printer device */
	if (arg >= argc) usage();
	if (!strcmp( argv[arg], "-v" ))
		report = 1, ++arg;
	else
		printer = argv[arg++];

	/* if no printer name yet, then here */
	if (!printer) {
		if (arg >= argc) usage();
		printer = argv[arg++];
	}

	/* open printer */
	if ((PrinterFd = open( printer, O_RDONLY )) < 0) {
		perror( printer );
		exit( 1 );
	}
	
	if (arg >= argc) {
		if (report) {
			for( i = 0; i < N_CMDS; ++i )
				Cmds[i].report();
		}
		else
			usage();
	}
	else {
		if ((argc - arg) % 2 != 0) usage();
		
		for( ; arg+1 < argc; arg += 2 ) {
			for( i = 0; i < N_CMDS; ++i ) {
				if (!strcmp( argv[arg], Cmds[i].cmd ))
					break;
			}
			if (i == N_CMDS) usage();
			Cmds[i].set( argv[arg+1] );
			if (report)
				Cmds[i].report();
		}
	}

	close( PrinterFd );
	return( 0 );
}


void usage( void )

{
	fprintf( stderr, "Usage: setSLM [-v] <printer> [pagesize <w>x<h>] "
			 "[mfeed on|off]\n" );
	exit( 1 );
}


void set_pagesize( const char *arg )

{	struct SLM_paper_size	psize;
	
	if (sscanf( arg, "%dx%d", &psize.width, &psize.height ) != 2)
		usage();
	if (ioctl( PrinterFd, SLMIOSPSIZE, &psize )) {
		perror( "ioctl(SLMIOSPSIZE)" );
	}
}


void get_pagesize( void )

{	struct SLM_paper_size	psize;

	if (ioctl( PrinterFd, SLMIOGPSIZE, &psize )) {
		perror( "ioctl(SLMIOGPSIZE)" );
	}
	else {
		printf( "Paper Size: %d x %d pixels\n", psize.width, psize.height );
	}
}


void set_mfeed( const char *arg )

{	int		mfeed = 0;

	if (!strcmp( arg, "on" ))
		mfeed = 1;
	else if (!strcmp( arg, "off" ))
		mfeed = 0;
	else
		usage();
	
	if (ioctl( PrinterFd, SLMIOSMFEED, mfeed )) {
		perror( "ioctl(SLMIOSMFEED)" );
	}
}


void get_mfeed( void )

{	int		mfeed;

	if (ioctl( PrinterFd, SLMIOGMFEED, &mfeed )) {
		perror( "ioctl(SLMIOGMFEED)" );
	}
	else {
		printf( "Manual Feed: %s\n", mfeed ? "on" : "off" );
	}
}


